package webutility

import (
	"net/http"
	"time"

	"git.to-net.rs/marko.tikvic/gologger"
)

func SetHeaders(h http.HandlerFunc) http.HandlerFunc {
	return func(w http.ResponseWriter, req *http.Request) {
		SetDefaultHeaders(w)
		if req.Method == http.MethodOptions {
			return
		}
		h(w, req)
	}
}

func ParseForm(h http.HandlerFunc) http.HandlerFunc {
	return func(w http.ResponseWriter, req *http.Request) {
		err := req.ParseForm()
		if err != nil {
			BadRequest(w, req, err.Error())
			return
		}
		h(w, req)
	}
}

func ParseMultipartForm(h http.HandlerFunc) http.HandlerFunc {
	return func(w http.ResponseWriter, req *http.Request) {
		err := req.ParseMultipartForm(32 << 20)
		if err != nil {
			BadRequest(w, req, err.Error())
			return
		}
		h(w, req)
	}
}

var trafficLogger *gologger.Logger

func EnableLogging(log string) error {
	var err error
	trafficLogger, err = gologger.New(log, gologger.MaxLogSize5MB)
	return err
}

func Log(h http.HandlerFunc) http.HandlerFunc {
	return func(w http.ResponseWriter, req *http.Request) {
		t1 := time.Now()

		in := trafficLogger.RequestLog(req, "")

		wRec := WrapWithStatusRecorder(w)
		h(wRec, req)

		t2 := time.Now()

		out := trafficLogger.ResponseLog(wRec.Status(), t2.Sub(t1), 0)

		trafficLogger.LogHTTPTraffic(in, out)
	}
}

func Auth(roles string, h http.HandlerFunc) http.HandlerFunc {
	return func(w http.ResponseWriter, req *http.Request) {
		if _, err := AuthCheck(req, roles); err != nil {
			Unauthorized(w, req, err.Error())
			return
		}
		h(w, req)
	}
}
